@@ -27,13 +27,14 @@
 #define EFFECT_END_BLOCK 0x02
 #define EFFECT_PLAYSEQ_POSITION 0x03
 #define EFFECT_STOP 0x04
-#define EFFECT_MIDI_COMMAND 0x05
+#define EFFECT_SEND_SYSEX 0x08
 #define EFFECT_NOTE_DELAY 0x0b
 #define EFFECT_VELOCITY 0x0c
 #define EFFECT_CHANNEL_PRESSURE 0x0d
 #define EFFECT_TPL 0x0e
 #define EFFECT_TEMPO 0x0f
 #define EFFECT_NOT_DEFINED 0x10
+#define EFFECT_MIDI_CONTROLLERS 0x80
 
 void play_song();
 void continue_song();
