@@ -26,6 +26,7 @@
 #include "playseq.h"
 #include "block.h"
 #include "instrument.h"
+#include "midi.h"
 
 struct song {
   /* Name of the song */
@@ -45,7 +46,7 @@ struct song {
   /* Block array */
   struct block **blocks;
   /* Number of instruments */
-  unsigned short numinstruments;
+  unsigned char numinstruments;
   /* Instrument array */
   struct instrument **instruments;
   /* Maximum number of tracks */
@@ -54,6 +55,10 @@ struct song {
   unsigned char *trackvolumes;
   /* Master volume */
   unsigned char mastervolume;
+  /* Number of System Exclusive messages */
+  unsigned short numsysexes;
+  /* System Exclusive messages */
+  struct sysex **sysexes;
 };
 
 /* Allocates a new song structure and initializes it to default values */
@@ -72,6 +77,10 @@ void song_delete_playseq(struct song *, int);
 void song_insert_section(struct song *, int);
 /* Deletes a section from the given position of the section array */
 void song_delete_section(struct song *, int);
+/* Inserts a new SysEx in the SysEx array in the given position */
+void song_insert_sysex(struct song *, int);
+/* Deletes a SysEx from the given position of the SysEx array */
+void song_delete_sysex(struct song *, int);
 /* Sets a section in the given position to point somewhere */
 void song_set_section(struct song *, int, int);
 /* Sets the number of ticks per line for a song */
