@@ -52,18 +52,30 @@ GtkWidget *gui_info_song, *gui_info_section, *gui_info_playseq;
 GtkWidget *gui_info_block1, *gui_info_block2;
 GtkWidget *gui_info_status, *gui_info_timer;
 
+/* SysEx list window GTK widgets */
+GtkWidget *gui_sysexlist_window, *gui_sysexlist_clist, *gui_sysexlist_name;
+GtkWidget *gui_sysexlist_length, *gui_sysexlist_current;
+GtkWidget *gui_sysexlist_name_hbox, *gui_sysexlist_length_hbox;
+GtkWidget *gui_sysexlist_sendreceive_hbox, *gui_sysexlist_delete_button;
+/* Current SysEx number */
+int gui_sysexlist_sysex;
+unsigned char gui_sysexlist_autostop;
+
 /* Section List window GTK widgets */
 GtkWidget *gui_sectionlist_window, *gui_sectionlist_current;
 GtkWidget *gui_sectionlist_playseq, *gui_sectionlist_clist;
+GtkWidget *gui_sectionlist_delete_button;
 
 /* Playing Sequence window GTK widgets */
 GtkWidget *gui_playseq_window, *gui_playseq_current, *gui_playseq_block;
 GtkWidget *gui_playseq_clist, *gui_playseq_name;
+GtkWidget *gui_playseq_delete_button;
 
 /* Block List window GTK widgets */
 GtkWidget *gui_blocklist_window, *gui_blocklist_current, *gui_blocklist_clist;
 GtkWidget *gui_blocklist_name, *gui_blocklist_tracks;
 GtkWidget *gui_blocklist_length, *gui_blocklist_effectpages;
+GtkWidget *gui_blocklist_delete_button;
 
 /* Instrument properties window GTK widgets */
 GtkWidget *gui_instrument_window, *gui_instrument_name, *gui_instrument_volume;
@@ -140,6 +152,7 @@ void gui_file_saveas_clicked(GtkFileSelection *selector, gpointer user_data) {
 
 void gui_create() {
   GtkWidget *w1, *w2, *w3, *w4, *mainvbox, *tophbox, *instrvbox;
+  char *sysexlist_titles[2]={ "SysEx", "Name" };
   char *sectionlist_titles[3]={ "Section", "Playing Sequence",
 				"Playing Sequence Name" };
   char *playseq_titles[3]={ "Position", "Block", "Block Name" };
@@ -194,6 +207,7 @@ void gui_create() {
 		     GTK_SIGNAL_FUNC(gtk_main_quit), NULL);
 
   w2=gtk_menu_item_new_with_label("Edit");
+  gtk_widget_set_sensitive(w2, FALSE);
   gtk_menu_bar_append(GTK_MENU_BAR(w1), w2);
   w3=gtk_menu_new();
   gtk_menu_item_set_submenu(GTK_MENU_ITEM(w2), w3);
@@ -216,6 +230,10 @@ void gui_create() {
   gtk_menu_append(GTK_MENU(w3), w4);
   gtk_signal_connect(GTK_OBJECT(w4), "activate",
 		     GTK_SIGNAL_FUNC(gui_trackvolumes_show), NULL);
+  w4=gtk_menu_item_new_with_label("SysEx List... AltGr+S");
+  gtk_menu_append(GTK_MENU(w3), w4);
+  gtk_signal_connect(GTK_OBJECT(w4), "activate",
+		     GTK_SIGNAL_FUNC(gui_sysexlist_show), NULL);
 
   w2=gtk_menu_item_new_with_label("Block");
   gtk_menu_bar_append(GTK_MENU_BAR(w1), w2);
@@ -395,7 +413,7 @@ void gui_create() {
   gtk_box_pack_start(GTK_BOX(w1), w2, TRUE, TRUE, 4);
   w3=gtk_hbox_new(FALSE, 4);
   gtk_container_add(GTK_CONTAINER(w2), w3);
-  gui_info_block1=gtk_label_new("Block 0/0");
+  gui_info_block1=gtk_label_new("Block 1/1");
   gui_info_block2=gtk_label_new("");
   gtk_box_pack_start(GTK_BOX(w3), gui_info_block1, TRUE, TRUE, 0);
   gtk_box_pack_start(GTK_BOX(w3), gui_info_block2, TRUE, TRUE, 0);
@@ -458,7 +476,7 @@ void gui_create() {
 		     GTK_SIGNAL_FUNC(gui_tempo_hide), NULL);
   gtk_box_pack_start(GTK_BOX(w1), w3, FALSE, FALSE, 0);
 
-  /* Tempo window */
+  /* Track volumes window */
   gui_trackvolumes_window=gtk_window_new(GTK_WINDOW_TOPLEVEL);
   gtk_container_set_border_width(GTK_CONTAINER(gui_trackvolumes_window), 4);
   gtk_window_set_title(GTK_WINDOW(gui_trackvolumes_window), "Track Volumes");
@@ -482,11 +500,109 @@ void gui_create() {
 		     GTK_SIGNAL_FUNC(gui_trackvolumes_hide), NULL);
   gtk_box_pack_start(GTK_BOX(w1), w3, FALSE, FALSE, 0);
 
+  /* Section list window */
+  gui_sysexlist_window=gtk_window_new(GTK_WINDOW_TOPLEVEL);
+  gtk_container_set_border_width(GTK_CONTAINER(gui_sysexlist_window), 4);
+  gtk_window_set_title(GTK_WINDOW(gui_sysexlist_window),
+		       "System Exclusive Message List");
+  gtk_widget_add_events(GTK_WIDGET(gui_sysexlist_window),
+			GDK_KEY_RELEASE_MASK);
+  gtk_signal_connect(GTK_OBJECT(gui_sysexlist_window), "key_press_event",
+		     GTK_SIGNAL_FUNC(keyboard_event), (gpointer)1);
+  gtk_signal_connect(GTK_OBJECT(gui_sysexlist_window), "key_release_event",
+		     GTK_SIGNAL_FUNC(keyboard_event), (gpointer)0);
+
+  w1=gtk_vbox_new(FALSE, 0);
+  gtk_container_add(GTK_CONTAINER(gui_sysexlist_window), w1);
+  w2=gtk_scrolled_window_new(NULL, NULL);
+  gtk_box_pack_start(GTK_BOX(w1), w2, TRUE, TRUE, 0);
+  gtk_container_set_border_width(GTK_CONTAINER(w2), 0);
+  gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(w2), GTK_POLICY_NEVER,
+				 GTK_POLICY_AUTOMATIC);
+  gui_sysexlist_clist=gtk_clist_new_with_titles(2, sysexlist_titles);
+  gtk_clist_set_column_width(GTK_CLIST(gui_sysexlist_clist), 0, 30);
+  gtk_clist_set_column_width(GTK_CLIST(gui_sysexlist_clist), 1, 30);
+  gtk_clist_set_column_auto_resize(GTK_CLIST(gui_sysexlist_clist), 0, TRUE);
+  gtk_clist_set_column_auto_resize(GTK_CLIST(gui_sysexlist_clist), 1, TRUE);
+  gtk_clist_set_selection_mode(GTK_CLIST(gui_sysexlist_clist),
+			       GTK_SELECTION_BROWSE);
+  gtk_signal_connect(GTK_OBJECT(gui_sysexlist_clist), "select_row",
+		     GTK_SIGNAL_FUNC(gui_sysexlist_select_row), NULL);
+  gtk_container_add(GTK_CONTAINER(w2), gui_sysexlist_clist);
+
+  gui_sysexlist_current=gtk_frame_new("SysEx 0/0");
+  gtk_frame_set_shadow_type(GTK_FRAME(gui_sysexlist_current),
+			    GTK_SHADOW_ETCHED_IN);
+  gtk_box_pack_start(GTK_BOX(w1), gui_sysexlist_current, FALSE, FALSE, 4);
+  w2=gtk_vbox_new(FALSE, 0);
+  gtk_container_add(GTK_CONTAINER(gui_sysexlist_current), w2);
+
+  gui_sysexlist_name_hbox=gtk_hbox_new(FALSE, 0);
+  gtk_box_pack_start(GTK_BOX(w2), gui_sysexlist_name_hbox, FALSE, FALSE, 4);
+  w4=gtk_label_new("Name");
+  gtk_box_pack_start(GTK_BOX(gui_sysexlist_name_hbox), w4, FALSE, FALSE, 4);
+  gtk_label_set_justify(GTK_LABEL(w4), GTK_JUSTIFY_RIGHT);
+  gui_sysexlist_name=gtk_entry_new();
+  gtk_signal_connect(GTK_OBJECT(gui_sysexlist_name), "activate",
+		     GTK_SIGNAL_FUNC(gui_sysexlist_activate_name), gui_sysexlist_name);
+  gtk_box_pack_start(GTK_BOX(gui_sysexlist_name_hbox), gui_sysexlist_name, TRUE, TRUE, 4);
+
+  gui_sysexlist_length_hbox=gtk_hbox_new(FALSE, 0);
+  gtk_box_pack_start(GTK_BOX(w2), gui_sysexlist_length_hbox, FALSE, FALSE, 0);
+  w4=gtk_label_new("Length");
+  gtk_box_pack_start(GTK_BOX(gui_sysexlist_length_hbox), w4, FALSE, FALSE, 4);
+  gui_sysexlist_length=(GtkWidget *)gtk_adjustment_new(0, 0, 1024*1024,
+						       1, 1, 0);
+  gtk_signal_connect(GTK_OBJECT(gui_sysexlist_length), "value_changed",
+		     GTK_SIGNAL_FUNC(gui_sysexlist_length_changed), NULL);
+  w4=gtk_spin_button_new(GTK_ADJUSTMENT(gui_sysexlist_length), 1, 0);
+  gtk_box_pack_start(GTK_BOX(gui_sysexlist_length_hbox), w4, TRUE, TRUE, 4);
+
+  gui_sysexlist_sendreceive_hbox=gtk_hbox_new(FALSE, 0);
+  gtk_box_pack_start(GTK_BOX(w2), gui_sysexlist_sendreceive_hbox, FALSE, FALSE, 4);
+  w4=gtk_button_new_with_label("Send");
+  gtk_signal_connect(GTK_OBJECT(w4), "clicked",
+		     GTK_SIGNAL_FUNC(gui_sysexlist_send), NULL);
+  gtk_box_pack_start(GTK_BOX(gui_sysexlist_sendreceive_hbox), w4,
+		     TRUE, TRUE, 4);
+  w4=gtk_button_new_with_label("Receive");
+  gtk_signal_connect(GTK_OBJECT(w4), "clicked",
+		     GTK_SIGNAL_FUNC(gui_sysexlist_receive), NULL);
+  gtk_box_pack_start(GTK_BOX(gui_sysexlist_sendreceive_hbox), w4,
+		     TRUE, TRUE, 0);
+  w4=gtk_check_button_new_with_label("Auto stop");
+  gtk_signal_connect(GTK_OBJECT(w4), "toggled",
+		     GTK_SIGNAL_FUNC(gui_main_toggled),
+		     (gpointer)&gui_sysexlist_autostop);
+  gtk_box_pack_start(GTK_BOX(gui_sysexlist_sendreceive_hbox), w4,
+		     TRUE, TRUE, 4);
+
+  w3=gtk_hbox_new(FALSE, 0);
+  gtk_box_pack_start(GTK_BOX(w2), w3, FALSE, FALSE, 4);
+  w4=gtk_button_new_with_label("Insert New");
+  gtk_signal_connect(GTK_OBJECT(w4), "clicked",
+		     GTK_SIGNAL_FUNC(gui_sysexlist_insert), NULL);
+  gtk_box_pack_start(GTK_BOX(w3), w4, TRUE, TRUE, 4);
+  w4=gtk_button_new_with_label("Append New");
+  gtk_signal_connect(GTK_OBJECT(w4), "clicked",
+		     GTK_SIGNAL_FUNC(gui_sysexlist_append), NULL);
+  gtk_box_pack_start(GTK_BOX(w3), w4, TRUE, TRUE, 0);
+  gui_sysexlist_delete_button=gtk_button_new_with_label("Delete");
+  gtk_signal_connect(GTK_OBJECT(gui_sysexlist_delete_button), "clicked",
+		     GTK_SIGNAL_FUNC(gui_sysexlist_delete), NULL);
+  gtk_box_pack_start(GTK_BOX(w3), gui_sysexlist_delete_button, TRUE, TRUE, 4);
+
+  w2=gtk_button_new_with_label("Close");
+  gtk_signal_connect(GTK_OBJECT(w2), "clicked",
+		     GTK_SIGNAL_FUNC(gui_sysexlist_hide), NULL);
+  gtk_box_pack_start(GTK_BOX(w1), w2, FALSE, FALSE, 0);
+
   /* Section list window */
   gui_sectionlist_window=gtk_window_new(GTK_WINDOW_TOPLEVEL);
   gtk_container_set_border_width(GTK_CONTAINER(gui_sectionlist_window), 4);
   gtk_window_set_title(GTK_WINDOW(gui_sectionlist_window), "Section List");
-  gtk_widget_add_events(GTK_WIDGET(gui_sectionlist_window), GDK_KEY_RELEASE_MASK);
+  gtk_widget_add_events(GTK_WIDGET(gui_sectionlist_window),
+			GDK_KEY_RELEASE_MASK);
   gtk_signal_connect(GTK_OBJECT(gui_sectionlist_window), "key_press_event",
 		     GTK_SIGNAL_FUNC(keyboard_event), (gpointer)1);
   gtk_signal_connect(GTK_OBJECT(gui_sectionlist_window), "key_release_event",
@@ -542,10 +658,10 @@ void gui_create() {
   gtk_signal_connect(GTK_OBJECT(w4), "clicked",
 		     GTK_SIGNAL_FUNC(gui_sectionlist_append), NULL);
   gtk_box_pack_start(GTK_BOX(w3), w4, TRUE, TRUE, 0);
-  w4=gtk_button_new_with_label("Delete");
-  gtk_signal_connect(GTK_OBJECT(w4), "clicked",
+  gui_sectionlist_delete_button=gtk_button_new_with_label("Delete");
+  gtk_signal_connect(GTK_OBJECT(gui_sectionlist_delete_button), "clicked",
 		     GTK_SIGNAL_FUNC(gui_sectionlist_delete), NULL);
-  gtk_box_pack_start(GTK_BOX(w3), w4, TRUE, TRUE, 4);
+  gtk_box_pack_start(GTK_BOX(w3), gui_sectionlist_delete_button, TRUE, TRUE, 4);
 
   w2=gtk_button_new_with_label("Close");
   gtk_signal_connect(GTK_OBJECT(w2), "clicked",
@@ -590,7 +706,7 @@ void gui_create() {
 
   w3=gtk_hbox_new(FALSE, 0);
   gtk_box_pack_start(GTK_BOX(w2), w3, FALSE, FALSE, 0);
-  gui_playseq_block=gtk_label_new("Block 0/0");
+  gui_playseq_block=gtk_label_new("Block 1/1");
   gtk_box_pack_start(GTK_BOX(w3), gui_playseq_block, FALSE, FALSE, 4);
   w4=gtk_button_new_with_label("<");
   gtk_signal_connect(GTK_OBJECT(w4), "clicked",
@@ -616,14 +732,6 @@ void gui_create() {
 		     GTK_SIGNAL_FUNC(gui_playseq_delete), NULL);
   gtk_box_pack_start(GTK_BOX(w3), w4, TRUE, TRUE, 4);
 
-#ifdef hrr
-  gui_playseq_playseq=gtk_frame_new("Playing Sequence 0/0");
-  gtk_frame_set_shadow_type(GTK_FRAME(gui_playseq_playseq),
-			    GTK_SHADOW_ETCHED_IN);
-  gtk_box_pack_start(GTK_BOX(w1), gui_playseq_playseq, FALSE, FALSE, 4);
-  w2=gtk_vbox_new(FALSE, 0);
-  gtk_container_add(GTK_CONTAINER(gui_playseq_playseq), w2);
-#endif
   w3=gtk_hbox_new(FALSE, 0);
   gtk_box_pack_start(GTK_BOX(w1), w3, FALSE, FALSE, 4);
   w4=gtk_label_new("Name");
@@ -643,10 +751,10 @@ void gui_create() {
   gtk_signal_connect(GTK_OBJECT(w4), "clicked",
 		     GTK_SIGNAL_FUNC(gui_playseq_appendnew), NULL);
   gtk_box_pack_start(GTK_BOX(w3), w4, TRUE, TRUE, 0);
-  w4=gtk_button_new_with_label("Delete");
-  gtk_signal_connect(GTK_OBJECT(w4), "clicked",
+  gui_playseq_delete_button=gtk_button_new_with_label("Delete");
+  gtk_signal_connect(GTK_OBJECT(gui_playseq_delete_button), "clicked",
 		     GTK_SIGNAL_FUNC(gui_playseq_delete), NULL);
-  gtk_box_pack_start(GTK_BOX(w3), w4, TRUE, TRUE, 4);
+  gtk_box_pack_start(GTK_BOX(w3), gui_playseq_delete_button, TRUE, TRUE, 4);
 
   w2=gtk_button_new_with_label("Close");
   gtk_signal_connect(GTK_OBJECT(w2), "clicked",
@@ -731,10 +839,10 @@ void gui_create() {
   gtk_signal_connect(GTK_OBJECT(w4), "clicked",
 		     GTK_SIGNAL_FUNC(gui_blocklist_append), NULL);
   gtk_box_pack_start(GTK_BOX(w3), w4, TRUE, TRUE, 0);
-  w4=gtk_button_new_with_label("Delete");
-  gtk_signal_connect(GTK_OBJECT(w4), "clicked",
+  gui_blocklist_delete_button=gtk_button_new_with_label("Delete");
+  gtk_signal_connect(GTK_OBJECT(gui_blocklist_delete_button), "clicked",
 		     GTK_SIGNAL_FUNC(gui_blocklist_delete), NULL);
-  gtk_box_pack_start(GTK_BOX(w3), w4, TRUE, TRUE, 4);
+  gtk_box_pack_start(GTK_BOX(w3), gui_blocklist_delete_button, TRUE, TRUE, 4);
 
   w2=gtk_button_new_with_label("Close");
   gtk_signal_connect(GTK_OBJECT(w2), "clicked",
@@ -848,10 +956,20 @@ void gui_main_show() {
 }
 
 void gui_tempo_refresh() {
+  char title[76];
+
+  /* Tempo window sliders */
   gtk_adjustment_set_value(GTK_ADJUSTMENT(gui_tempo_adjustment_tempo),
 			   editor.song->tempo);
   gtk_adjustment_set_value(GTK_ADJUSTMENT(gui_tempo_adjustment_tpl),
 			   editor.song->ticksperline);
+
+  /* Window title bar */
+  snprintf(title, 76,
+	   "Tutka: Block %d/%d - Cmd Page %d/%d - Tempo %d - TPL %d",
+	   editor.block, editor.song->numblocks-1, 1, 1,
+	   editor.song->tempo, editor.song->ticksperline);
+  gtk_window_set_title(GTK_WINDOW(gui_main_window), title); 
 }
 
 void gui_tempo_show() {
@@ -918,7 +1036,7 @@ void gui_info_refresh() {
 	   editor.song->numsections);
   snprintf(playseq, 9+8, "Position %d/%d", editor.playseqpos+1,
 	   editor.song->playseqs[editor.playseq]->length);
-  snprintf(block, 6+9, "Block %d/%d:", editor.block, editor.song->numblocks-1);
+  snprintf(block, 6+9, "Block %d/%d:", editor.block+1, editor.song->numblocks);
   gtk_label_set_text(GTK_LABEL(gui_info_section), section);
   gtk_label_set_text(GTK_LABEL(gui_info_playseq), playseq);
   gtk_label_set_text(GTK_LABEL(gui_info_block1), block);
@@ -927,12 +1045,14 @@ void gui_info_refresh() {
     gtk_label_set_text(GTK_LABEL(gui_info_block2),
 		       editor.song->blocks[editor.block]->name);
 
+  /* Window title bar */
   snprintf(title, 76,
 	   "Tutka: Block %d/%d - Cmd Page %d/%d - Tempo %d - TPL %d",
-	   editor.block, editor.song->numblocks-1, 0, 0,
+	   editor.block+1, editor.song->numblocks, 1, 1,
 	   editor.song->tempo, editor.song->ticksperline);
   gtk_window_set_title(GTK_WINDOW(gui_main_window), title); 
 
+  /* Status bar */
   switch(editor.mode) {
   case MODE_PLAY_SONG:
     gtk_label_set_text(GTK_LABEL(gui_info_status), "Playing song");
@@ -990,7 +1110,131 @@ void gui_instrument_transpose_changed(GtkAdjustment *adjustment,
 
 void gui_instrument_hold_changed(GtkAdjustment *adjustment,
 				 gpointer user_data) {
-  editor.song->instruments[editor.instrument]->transpose=(unsigned char)adjustment->value;
+  editor.song->instruments[editor.instrument]->hold=(unsigned char)adjustment->value;
+}
+
+void gui_sysexlist_show() {
+  gtk_window_set_default_size(GTK_WINDOW(gui_sysexlist_window), 200, 300);
+  gtk_widget_show_all(gui_sysexlist_window);
+}
+
+void gui_sysexlist_hide() {
+  gtk_widget_hide(gui_sysexlist_window);
+}
+
+void gui_sysexlist_refresh() {
+  int i, sysex;
+  char number[4];
+  char *columns[2];
+
+  if(editor.song->numsysexes==0)
+    gui_sysexlist_sysex=-1;
+
+  sysex=gui_sysexlist_sysex;
+
+  /* Recreate the SysEx list */
+  gtk_clist_freeze(GTK_CLIST(gui_sysexlist_clist));
+  gtk_clist_clear(GTK_CLIST(gui_sysexlist_clist));
+  columns[0]=number;
+  for(i=0; i<editor.song->numsysexes; i++) {
+    snprintf(number, 4, "%3d", i);
+    columns[1]=editor.song->sysexes[i]->name;
+    gtk_clist_append(GTK_CLIST(gui_sysexlist_clist), columns);
+  }
+  gtk_clist_thaw(GTK_CLIST(gui_sysexlist_clist));
+
+  /* Move back to the previously selected SysEx and make widgets sensitive */
+  if(gui_sysexlist_sysex!=-1) {
+    gtk_clist_select_row(GTK_CLIST(gui_sysexlist_clist), sysex, 0);
+    gtk_clist_moveto(GTK_CLIST(gui_sysexlist_clist), sysex, 0, 0, 0);
+    gtk_widget_set_sensitive(gui_sysexlist_name_hbox, TRUE);
+    gtk_widget_set_sensitive(gui_sysexlist_length_hbox, TRUE);
+    gtk_widget_set_sensitive(gui_sysexlist_sendreceive_hbox, TRUE);
+    gtk_widget_set_sensitive(gui_sysexlist_delete_button, TRUE);
+  } else {
+    gtk_frame_set_label(GTK_FRAME(gui_sysexlist_current), "SysEx 0/0");
+    gtk_widget_set_sensitive(gui_sysexlist_name_hbox, FALSE);
+    gtk_widget_set_sensitive(gui_sysexlist_length_hbox, FALSE);
+    gtk_widget_set_sensitive(gui_sysexlist_sendreceive_hbox, FALSE);
+    gtk_widget_set_sensitive(gui_sysexlist_delete_button, FALSE);
+  }
+}
+
+void gui_sysexlist_select_row(GtkWidget *list, gint row, gint column,
+			      GdkEventButton *event, gpointer data) {
+  char sysex[6+8];
+  gui_sysexlist_sysex=row;
+
+  /* Show the number of the selected SysEx message */
+  snprintf(sysex, 6+8, "SysEx %d/%d", gui_sysexlist_sysex+1,
+	   editor.song->numsysexes);
+  gtk_frame_set_label(GTK_FRAME(gui_sysexlist_current), sysex);
+
+  /* Show name of the selected SysEx message */
+  if(editor.song->sysexes[gui_sysexlist_sysex]->name)
+    gtk_entry_set_text(GTK_ENTRY(gui_sysexlist_name),
+		       editor.song->sysexes[gui_sysexlist_sysex]->name);
+  else
+    gtk_entry_set_text(GTK_ENTRY(gui_sysexlist_name), "");
+
+  /* Show length of the selected SysEx message */
+  gtk_adjustment_set_value(GTK_ADJUSTMENT(gui_sysexlist_length),
+			   editor.song->sysexes[gui_sysexlist_sysex]->length);
+}
+
+void gui_sysexlist_activate_name(GtkWidget *widget, GtkWidget *entry) {
+  if(editor.song!=NULL && gui_sysexlist_sysex!=-1) {
+    char *name=gtk_entry_get_text(GTK_ENTRY(entry));
+    
+    if(editor.song->sysexes[gui_sysexlist_sysex]->name)
+      free(editor.song->sysexes[gui_sysexlist_sysex]->name);
+    
+    editor.song->sysexes[gui_sysexlist_sysex]->name=(char *)strdup(name);
+
+    gui_sysexlist_refresh();
+  }
+}
+
+void gui_sysexlist_insert() {
+  if(editor.song!=NULL) {
+    song_insert_sysex(editor.song, gui_sysexlist_sysex);
+    gui_sysexlist_refresh();
+  }
+}
+
+void gui_sysexlist_append() {
+  if(editor.song!=NULL) {
+    song_insert_sysex(editor.song, editor.song->numsysexes);
+    gui_sysexlist_refresh();
+  }
+}
+
+void gui_sysexlist_delete() {
+  if(editor.song!=NULL && gui_sysexlist_sysex!=-1) {
+    song_delete_sysex(editor.song, gui_sysexlist_sysex);
+    gui_sysexlist_refresh();
+  }
+}
+
+void gui_sysexlist_length_changed(GtkAdjustment *adjustment,
+				  gpointer user_data) {
+  if(editor.song!=NULL && gui_sysexlist_sysex!=-1)
+    sysex_set_length(editor.song->sysexes[gui_sysexlist_sysex],
+		     (unsigned int)adjustment->value);
+}
+
+void gui_sysexlist_send() {
+  if(editor.song!=NULL && gui_sysexlist_sysex!=-1) {
+    midi_system_exclusive(editor.song->sysexes[gui_sysexlist_sysex]->data,
+			  editor.song->sysexes[gui_sysexlist_sysex]->length);
+  }
+}
+
+void gui_sysexlist_receive() {
+  if(editor.song!=NULL && gui_sysexlist_sysex!=-1) {
+    midi_read_system_exclusive(editor.song->sysexes[gui_sysexlist_sysex],
+			       gui_sysexlist_autostop);
+  }
 }
 
 void gui_sectionlist_show() {
@@ -1009,20 +1253,28 @@ void gui_sectionlist_refresh() {
   char playseq[4];
   char *columns[3];
 
+  /* Recreate the section list */
   gtk_clist_freeze(GTK_CLIST(gui_sectionlist_clist));
   gtk_clist_clear(GTK_CLIST(gui_sectionlist_clist));
   columns[0]=position;
   columns[1]=playseq;
   for(i=0; i<editor.song->numsections; i++) {
-    snprintf(position, 4, "%0.3d", i+1);
-    snprintf(playseq, 4, "%0.3d", editor.song->sections[i]+1);
+    snprintf(position, 4, "%3d", i+1);
+    snprintf(playseq, 4, "%3d", editor.song->sections[i]+1);
     columns[2]=editor.song->playseqs[editor.song->sections[i]]->name;
     gtk_clist_append(GTK_CLIST(gui_sectionlist_clist), columns);
   }
-
   gtk_clist_thaw(GTK_CLIST(gui_sectionlist_clist));
+
+  /* Move back to the previously selected section */
   gtk_clist_select_row(GTK_CLIST(gui_sectionlist_clist), section, 0);
   gtk_clist_moveto(GTK_CLIST(gui_sectionlist_clist), section, 0, 0, 0);
+
+  /* Make the delete button sensitive/insensitive */
+  if(editor.song->numsections>1)
+    gtk_widget_set_sensitive(gui_sectionlist_delete_button, TRUE);
+  else
+    gtk_widget_set_sensitive(gui_sectionlist_delete_button, FALSE);
 }
 
 void gui_sectionlist_select_row(GtkWidget *list, gint row, gint column,
@@ -1116,9 +1368,9 @@ void gui_playseq_refresh() {
   columns[0]=position;
   columns[1]=block;
   for(i=0; i<editor.song->playseqs[editor.playseq]->length; i++) {
-    snprintf(position, 4, "%0.3d", i+1);
-    snprintf(block, 4, "%0.3d",
-	     editor.song->playseqs[editor.playseq]->blocknumbers[i]);
+    snprintf(position, 4, "%3d", i+1);
+    snprintf(block, 4, "%3d",
+	     editor.song->playseqs[editor.playseq]->blocknumbers[i]+1);
     columns[2]=editor.song->blocks[editor.song->playseqs[editor.playseq]->blocknumbers[i]]->name;
     gtk_clist_append(GTK_CLIST(gui_playseq_clist), columns);
   }
@@ -1129,13 +1381,19 @@ void gui_playseq_refresh() {
 
   snprintf(position, 9+8, "Position %d/%d", editor.playseqpos+1,
 	   editor.song->playseqs[editor.playseq]->length);
-  snprintf(block, 6+8, "Block %d/%d", editor.song->playseqs[editor.playseq]->blocknumbers[editor.playseqpos], editor.song->numblocks-1);
+  snprintf(block, 6+8, "Block %d/%d", editor.song->playseqs[editor.playseq]->blocknumbers[editor.playseqpos]+1, editor.song->numblocks);
   gtk_frame_set_label(GTK_FRAME(gui_playseq_current), position);
   gtk_label_set_text(GTK_LABEL(gui_playseq_block), block);
 
   snprintf(title, 24, "Playing Sequence %d/%d", editor.playseq+1,
 	   editor.song->numplayseqs);
   gtk_window_set_title(GTK_WINDOW(gui_playseq_window), title);
+
+  /* Make the delete button sensitive/insensitive */
+  if(editor.song->numplayseqs>1)
+    gtk_widget_set_sensitive(gui_playseq_delete_button, TRUE);
+  else
+    gtk_widget_set_sensitive(gui_playseq_delete_button, FALSE);
 }
 
 void gui_playseq_select_row(GtkWidget *list, gint row, gint column,
@@ -1147,7 +1405,7 @@ void gui_playseq_select_row(GtkWidget *list, gint row, gint column,
 
   snprintf(position, 9+8, "Position %d/%d", editor.playseqpos+1,
 	   editor.song->playseqs[editor.playseq]->length);
-  snprintf(block, 6+8, "Block %d/%d", editor.song->playseqs[editor.playseq]->blocknumbers[editor.playseqpos], editor.song->numblocks-1);
+  snprintf(block, 6+8, "Block %d/%d", editor.song->playseqs[editor.playseq]->blocknumbers[editor.playseqpos]+1, editor.song->numblocks);
   gtk_frame_set_label(GTK_FRAME(gui_playseq_current), position);
   gtk_label_set_text(GTK_LABEL(gui_playseq_block), block);
 
@@ -1236,55 +1494,59 @@ void gui_blocklist_refresh() {
   char number[4];
   char *columns[2];
 
+  /* Recreate the block list */
   gtk_clist_freeze(GTK_CLIST(gui_blocklist_clist));
   gtk_clist_clear(GTK_CLIST(gui_blocklist_clist));
   columns[0]=number;
   for(i=0; i<editor.song->numblocks; i++) {
-    snprintf(number, 4, "%0.3d", i);
+    snprintf(number, 4, "%3d", i+1);
     columns[1]=editor.song->blocks[i]->name;
     gtk_clist_append(GTK_CLIST(gui_blocklist_clist), columns);
   }
-
   gtk_clist_thaw(GTK_CLIST(gui_blocklist_clist));
+
+  /* Move back to the previously selected block */
   gtk_clist_select_row(GTK_CLIST(gui_blocklist_clist), block, 0);
   gtk_clist_moveto(GTK_CLIST(gui_blocklist_clist), block, 0, 0, 0);
 
+  /* Make the delete button sensitive/insensitive */
+  if(editor.song->numblocks>1)
+    gtk_widget_set_sensitive(gui_blocklist_delete_button, TRUE);
+  else
+    gtk_widget_set_sensitive(gui_blocklist_delete_button, FALSE);
+}
+
+void gui_blocklist_select_row(GtkWidget *list, gint row, gint column,
+		      GdkEventButton *event, gpointer data) {
+  char block[6+8];
+  editor.block=row;
+
+  /* Show the number of the selected block */
+  snprintf(block, 6+8, "Block %d/%d", editor.block+1,
+	   editor.song->numblocks);
+  gtk_frame_set_label(GTK_FRAME(gui_blocklist_current), block);
+
+  /* Show the name of the selected block */
+  if(editor.song->blocks[editor.block]->name)
+    gtk_entry_set_text(GTK_ENTRY(gui_blocklist_name),
+		       editor.song->blocks[editor.block]->name);
+  else
+    gtk_entry_set_text(GTK_ENTRY(gui_blocklist_name), "");
+ 
+  /* Show the properties of the selected block */
   gtk_adjustment_set_value(GTK_ADJUSTMENT(gui_blocklist_tracks),
 			   editor.song->blocks[editor.block]->tracks);
   gtk_adjustment_set_value(GTK_ADJUSTMENT(gui_blocklist_length),
 			   editor.song->blocks[editor.block]->length);
   gtk_adjustment_set_value(GTK_ADJUSTMENT(gui_blocklist_effectpages),
 			   editor.song->blocks[editor.block]->effectpages);
-}
-
-void gui_blocklist_select_row(GtkWidget *list, gint row, gint column,
-		      GdkEventButton *event, gpointer data) {
-  char block[6+8];
-
-  editor.block=row;
-
-  if(editor.song->blocks[row]->name)
-    gtk_entry_set_text(GTK_ENTRY(gui_blocklist_name),
-		       editor.song->blocks[row]->name);
-  else
-    gtk_entry_set_text(GTK_ENTRY(gui_blocklist_name), "");
-
-  snprintf(block, 6+8, "Block %d/%d", editor.block,
-	   editor.song->numblocks-1);
-
-  gtk_frame_set_label(GTK_FRAME(gui_blocklist_current), block);
 
+  /* Refresh the rest of the GUI (info, tracker) */
   gui_info_refresh();
-  tracker_set_pattern((Tracker *)gui_main_tracker, editor.song->blocks[row]);
+  tracker_set_pattern((Tracker *)gui_main_tracker,
+		      editor.song->blocks[editor.block]);
   tracker_set_num_channels((Tracker *)gui_main_tracker,
-			   editor.song->blocks[row]->tracks);
-  
-  gtk_adjustment_set_value(GTK_ADJUSTMENT(gui_blocklist_tracks),
-			   editor.song->blocks[row]->tracks);
-  gtk_adjustment_set_value(GTK_ADJUSTMENT(gui_blocklist_length),
-			   editor.song->blocks[row]->length);
-  gtk_adjustment_set_value(GTK_ADJUSTMENT(gui_blocklist_effectpages),
-			   editor.song->blocks[row]->effectpages);
+			   editor.song->blocks[editor.block]->tracks);
 }
 
 void gui_blocklist_activate_name(GtkWidget *widget, GtkWidget *entry) {
