@@ -395,6 +395,19 @@ int keyboard_event(GtkWidget *widget, GdkEventKey *event, gpointer data) {
       case GDK_ISO_Left_Tab:
 	tracker_step_cursor_channel((Tracker *)gui_main_tracker, -1);
 	break;
+      case GDK_Home:
+	tracker_set_patpos((Tracker *)gui_main_tracker, 0);
+	break;
+      case GDK_End:
+	tracker_set_patpos((Tracker *)gui_main_tracker,
+			   editor.song->blocks[editor.block]->length-1);
+	break;
+      case GDK_Page_Down:
+	tracker_step_cursor_row((Tracker *)gui_main_tracker, 8);
+	break;
+      case GDK_Page_Up:
+	tracker_step_cursor_row((Tracker *)gui_main_tracker, -8);
+	break;
       }
 
       /* Select an instrument if an instrument selection key was pressed */
