@@ -19,6 +19,7 @@
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
 
+#include "midi.h"
 #include "editor.h"
 
 extern struct editor editor;
@@ -115,8 +116,8 @@ void midi_pitch_wheel(unsigned char channel, unsigned short value) {
 
   if(editor.midi!=NULL) {
     data[0]=0xe0|channel;
-    data[1]=value&0x7f;
-    data[2]=(value>>7)&0x7f;
+    data[1]=(value>>7)&0x7f;
+    data[2]=value&0x7f;
     
     fwrite(data, 3, 1, editor.midi);
     fflush(editor.midi);
@@ -138,3 +139,93 @@ void midi_system_exclusive(unsigned char *message, unsigned short length) {
 
   free(data);
 }
+
+/* Receives a MIDI SysEx message */
+void midi_read_system_exclusive(struct sysex *sysex, int autostop) {
+  if(sysex!=NULL) {
+    int status=0, i=0;
+    
+    /* Read until a SysEx dump begins */
+    while(status!=0xf0 && status!=EOF)
+      status=fgetc(editor.midi);
+    
+    /* Return on error */
+    if(status==EOF)
+      return;
+    
+    /* Read until the buffer is full or to the end of the SysEx (autostop) */
+    while(i<sysex->length) {
+      status=fgetc(editor.midi);
+      
+      if(status==EOF)
+	break;
+      
+      sysex->data[i++]=status;
+      if(autostop==1 && sysex->data[i-1]==0xf7)
+	break;
+    }
+    
+    /* Remove unnecessary bytes if necessary */
+    if(i<sysex->length) {
+      sysex->length=i;
+      sysex->data=(unsigned char *)realloc(sysex->data, sysex->length);
+    }
+  }    
+}
+
+/* Sets the length of a MIDI SysEx message */
+void sysex_set_length(struct sysex *sysex, unsigned int length) {
+  if(sysex!=NULL) {
+    sysex->length=length;
+    sysex->data=(unsigned char *)realloc(sysex->data, sysex->length);
+  }
+}
+
+/* Parses a SysEx element in an XML file */
+struct sysex *sysex_parse(xmlDocPtr doc, xmlNsPtr ns, xmlNodePtr cur) {
+  struct sysex *sysex=NULL;
+  
+  if((!xmlStrcmp(cur->name, "sysex")) && (cur->ns==ns)) {
+    xmlNodePtr temp=cur->xmlChildrenNode;
+    char c[3];
+    int i;
+    unsigned int d;
+
+    /* Temporary string for hexadecimal parsing */
+    c[2]=0;
+
+    /* Allocate sysex */
+    sysex=(struct sysex *)calloc(1, sizeof(struct sysex));
+    sysex->name=xmlGetProp(cur, "name");
+    sysex->length=strlen(temp->content)/2;
+    sysex->data=(unsigned char *)calloc(sysex->length, sizeof(unsigned char));
+    for(i=0; i<sysex->length; i++) {
+      c[0]=temp->content[i*2];
+      c[1]=temp->content[i*2+1];
+      sscanf(c, "%X", &d);
+      sysex->data[i]=d;
+    }
+  } else
+    fprintf(stderr, "XML error: expected sysex, got %s\n", cur->name);
+
+  return sysex;
+}
+
+/* Saves a SysEx message to an XML file */
+void sysex_save(struct sysex *sysex, xmlNodePtr parent) {
+  char *c;
+  xmlNodePtr node, lb;
+  int i;
+
+  c=(char *)calloc(2*sysex->length+1, sizeof(char));
+  for(i=0; i<sysex->length; i++)
+    sprintf(c+(i*2), "%0.2X", sysex->data[i]);
+
+  node=xmlNewChild(parent, NULL, "sysex", c);
+  xmlSetProp(node, "name", sysex->name);
+
+  lb=xmlNewText("\n");
+  xmlAddChild(parent, lb);
+
+  free(c);
+}
