@@ -71,10 +71,10 @@ void continue_block() {
 
 /* Stops the player */
 void stop() {
+  player_stop_notes();
   if(editor.mode!=MODE_IDLE) {
     editor.mode=MODE_IDLE;
     player_stop();
-    player_stop_notes();
 
     /* Refresh the GUI accordingly
      * (FIX: This is called both from player and GUI... dangerous!) */
@@ -88,17 +88,51 @@ void player_next_tick() {
   int i, j;
   struct timeval tod;
   struct block *block=editor.song->blocks[editor.block];
-  unsigned char posteffect, postvalue;
+  unsigned char posteffect=0, postvalue=0;
 
   /* Play notes scheduled to be played */
   for(i=0; i<block->tracks; i++) {
     unsigned char volume=127;
     unsigned char notedelay=0;
-    /* Handle effects meant to be processed only at first tick */
+    unsigned char note=block->notes[(block->tracks*tracker->patpos+i)*2];
+    unsigned char instrument=block->notes[(block->tracks*tracker->patpos+i)*2+1];
+    /* Handle effects on all effect pages */
     for(j=0; j<block->effectpages; j++) {
       unsigned char effect=block->effects[j*block->tracks*block->length*2+(block->tracks*tracker->patpos+i)*2];
       unsigned char value=block->effects[j*block->tracks*block->length*2+(block->tracks*tracker->patpos+i)*2+1];
+      char midichannel=-1;
+
+      /* Check which MIDI channel the command will affect */
+      if(instrument!=0)
+	/* Instrument number defines MIDI channel */
+	midichannel=editor.song->instruments[instrument-1]->midichannel;
+      else {
+	/* Note playing defines MIDI channel */
+	if(editor.trackstatus[i*4+TRACK_STATUS_HOLD]!=-1)
+	  midichannel=editor.trackstatus[i*4+TRACK_STATUS_MIDI_CHANNEL];
+      }
+
       switch(effect) {
+      case EFFECT_PITCH_WHEEL:
+	/* Pitch wheel can be set if the MIDI channel is known */
+	if(midichannel!=-1) {
+	  if(value<0x80) {
+	    if(tick==0) {
+	      midi_pitch_wheel(midichannel, value);
+	      editor.midicontrollervalues[midichannel*VALUES+VALUES_PITCH_WHEEL]=value;
+	    }
+	  } else {
+	    if(tick<editor.song->ticksperline-1) {
+	      unsigned char delta=(value-0x80-editor.midicontrollervalues[midichannel*VALUES+VALUES_PITCH_WHEEL])/(editor.song->ticksperline-1);
+
+	      midi_pitch_wheel(midichannel, editor.midicontrollervalues[midichannel*VALUES+VALUES_PITCH_WHEEL]+tick*delta);
+	    } else {
+	      midi_pitch_wheel(midichannel, value-0x80);
+	      editor.midicontrollervalues[midichannel*VALUES+VALUES_PITCH_WHEEL]=value-0x80;
+	    }
+	  }
+	}
+	break;
       case EFFECT_END_BLOCK:
 	/* Only on last tick */
 	if(tick==editor.song->ticksperline-1) {
@@ -118,56 +152,129 @@ void player_next_tick() {
 	if(tick==editor.song->ticksperline-1)
 	  stop();
 	break;
-      case EFFECT_VELOCITY:
-	volume=value;
+      case EFFECT_SEND_SYSEX:
+	/* Only on first tick */
+	if(tick==0 && value<editor.song->numsysexes)
+	  midi_system_exclusive(editor.song->sysexes[value]->data,
+				editor.song->sysexes[value]->length);
 	break;
       case EFFECT_NOTE_DELAY:
 	notedelay=value;
 	break;
+      case EFFECT_VELOCITY:
+	if(note!=0) {
+	  volume=value;
+	  if(midichannel!=-1)
+	    editor.midicontrollervalues[midichannel*VALUES+VALUES_AFTERTOUCH]=value;
+	} else {
+	  /* Note playing defines MIDI channel */
+	  if(editor.trackstatus[i*4+TRACK_STATUS_HOLD]!=-1) {
+	    midichannel=editor.trackstatus[i*4+TRACK_STATUS_MIDI_CHANNEL];
+
+	    if(value<0x80) {
+	      if(tick==0) {
+		midi_aftertouch(midichannel, editor.trackstatus[i*4+TRACK_STATUS_NOTE], value);
+		editor.midicontrollervalues[midichannel*VALUES+VALUES_AFTERTOUCH]=value;
+	      }
+	    } else {
+	      if(tick<editor.song->ticksperline-1) {
+		unsigned char delta=(value-0x80-editor.midicontrollervalues[midichannel*VALUES+VALUES_AFTERTOUCH])/(editor.song->ticksperline-1);
+		
+		midi_aftertouch(midichannel, editor.trackstatus[i*4+TRACK_STATUS_NOTE], editor.midicontrollervalues[midichannel*VALUES+VALUES_AFTERTOUCH]+tick*delta);
+	      } else {
+		midi_aftertouch(midichannel, editor.trackstatus[i*4+TRACK_STATUS_NOTE], value-0x80);
+		editor.midicontrollervalues[midichannel*VALUES+VALUES_AFTERTOUCH]=value-0x80;
+	      }
+	    }
+	  }
+	}
+	break;
+      case EFFECT_CHANNEL_PRESSURE:
+	/* Channel pressure can be set if the MIDI channel is known */
+	if(midichannel!=-1) {
+	  if(value<0x80) {
+	    if(tick==0) {
+	      midi_channel_pressure(midichannel, value);
+	      editor.midicontrollervalues[midichannel*VALUES+VALUES_CHANNEL_PRESSURE]=value;
+	    }
+	  } else {
+	    if(tick<editor.song->ticksperline-1) {
+	      unsigned char delta=(value-0x80-editor.midicontrollervalues[midichannel*VALUES+VALUES_CHANNEL_PRESSURE])/(editor.song->ticksperline-1);
+
+	      midi_channel_pressure(midichannel, editor.midicontrollervalues[midichannel*VALUES+VALUES_CHANNEL_PRESSURE]+tick*delta);
+	    } else {
+	      midi_channel_pressure(midichannel, value-0x80);
+	      editor.midicontrollervalues[midichannel*VALUES+VALUES_CHANNEL_PRESSURE]=value-0x80;
+	    }
+	  }
+	}
+	break;
       case EFFECT_TPL:
 	song_set_tpl(editor.song, value);
 	gdk_threads_enter();
-	gui_info_refresh();
 	gui_tempo_refresh();
 	gdk_threads_leave();
 	break;
       case EFFECT_TEMPO:
 	song_set_tempo(editor.song, value);
 	gdk_threads_enter();
-	gui_info_refresh();
 	gui_tempo_refresh();
 	gdk_threads_leave();
 	break;
       }
+
+      /* Handle MIDI controllers */
+      if(effect>=EFFECT_MIDI_CONTROLLERS) {
+	/* MIDI controllers can be set if the MIDI channel is known */
+	if(midichannel!=-1) {
+	  if(value<0x80) {
+	    if(tick==0) {
+	      midi_controller(midichannel, effect-EFFECT_MIDI_CONTROLLERS,
+			      value);
+	      editor.midicontrollervalues[midichannel*VALUES+effect-EFFECT_MIDI_CONTROLLERS]=value;
+	    }
+	  } else {
+	    if(tick<editor.song->ticksperline-1) {
+	      unsigned char delta=(value-0x80-editor.midicontrollervalues[midichannel*VALUES+effect-EFFECT_MIDI_CONTROLLERS])/(editor.song->ticksperline-1);
+
+	      midi_controller(midichannel, effect-EFFECT_MIDI_CONTROLLERS,
+			      editor.midicontrollervalues[midichannel*VALUES+effect-EFFECT_MIDI_CONTROLLERS]+tick*delta);
+	    } else {
+	      midi_controller(midichannel, effect-EFFECT_MIDI_CONTROLLERS,
+			      value);
+	      editor.midicontrollervalues[midichannel*VALUES+effect-EFFECT_MIDI_CONTROLLERS]=value;
+	    }
+	  }
+	}
+      }
     }
     
-    if(block->notes[(block->tracks*tracker->patpos+i)*2]!=0 &&
-       tick==notedelay) {
+    if(note!=0 && tick==notedelay) {
       unsigned char hold=0;
-      unsigned char note=block->notes[(block->tracks*tracker->patpos+i)*2]-1;
-      unsigned short instrument=block->notes[(block->tracks*tracker->patpos+i)*2+1]-1;
-      
-      player_play_note(instrument, note, volume, i);
+      note--;
+
+      player_play_note(instrument-1, note, volume, i);
       
       if(instrument<editor.song->numinstruments &&
-	 editor.song->instruments[instrument]!=NULL)
+	 editor.song->instruments[instrument-1]!=NULL)
 	hold=editor.song->instruments[block->notes[(block->tracks*tracker->patpos+i)*2+1]-1]->hold;
       
       if(hold==0)
-	editor.trackstatus[i*4+3]=-1;
+	editor.trackstatus[i*4+TRACK_STATUS_HOLD]=-1;
       else
-	editor.trackstatus[i*4+3]=hold;
+	editor.trackstatus[i*4+TRACK_STATUS_HOLD]=hold;
     }
   }
 
   /* Decrement hold times of notes and stop notes that should be stopped */
   for(i=0; i<editor.song->maxtracks; i++) {
-    if(editor.trackstatus[i*4+3]!=-1) {
-      editor.trackstatus[i*4+3]--;
-      if(editor.trackstatus[i*4+3]==-1) {
-	midi_note_off(editor.trackstatus[i*4], editor.trackstatus[i*4+1],
+    if(editor.trackstatus[i*4+TRACK_STATUS_HOLD]!=-1) {
+      editor.trackstatus[i*4+TRACK_STATUS_HOLD]--;
+      if(editor.trackstatus[i*4+TRACK_STATUS_HOLD]==-1) {
+	midi_note_off(editor.trackstatus[i*4+TRACK_STATUS_MIDI_CHANNEL],
+		      editor.trackstatus[i*4+TRACK_STATUS_NOTE],
 		      127);
-	editor.trackstatus[i*4+1]=-1;
+	editor.trackstatus[i*4+TRACK_STATUS_NOTE]=-1;
       }
     }
   }
@@ -176,56 +283,47 @@ void player_next_tick() {
   tick++;
   tick%=editor.song->ticksperline;
 
-  switch(posteffect) {
-  case EFFECT_END_BLOCK:
-    player_next_playseq();
-    player_refresh_playseq_and_block();
-    gdk_threads_enter();
-    tracker_set_pattern(tracker, editor.song->blocks[editor.block]);
-    tracker_set_patpos(tracker, 0);
-    gui_info_refresh();
-    gui_blocklist_refresh();
-    gui_playseq_refresh();
-    gdk_threads_leave();
-    break;
-  case EFFECT_PLAYSEQ_POSITION:
-    editor.playseqpos=postvalue;
-    if(editor.playseqpos>=editor.song->playseqs[editor.playseq]->length){
-      editor.playseqpos=0;
-      player_next_section();
-    }
-    player_refresh_playseq_and_block();
-    gdk_threads_enter();
-    tracker_set_pattern(tracker, editor.song->blocks[editor.block]);
-    tracker_set_patpos(tracker, 0);
-    gui_info_refresh();
-    gui_blocklist_refresh();
-    gui_playseq_refresh();
-    gdk_threads_leave();
-    break;
-  default:
-    /* Advance in block if ticksperline ticks have passed */
-    if(tick==0) {
-      int newpos=tracker->patpos+1;
+  /* Advance and handle post effects if ticksperline ticks have passed */
+  if(tick==0) {
+    int newpos=tracker->patpos+1, changeblock=0;
+    switch(posteffect) {
+    case EFFECT_END_BLOCK:
+      newpos=postvalue;
+      player_next_playseq();
+      changeblock=1;
+      break;
+    case EFFECT_PLAYSEQ_POSITION:
+      newpos=0;
+      editor.playseqpos=postvalue;
+      if(editor.playseqpos>=editor.song->playseqs[editor.playseq]->length) {
+	editor.playseqpos=0;
+	player_next_section();
+      }
+      changeblock=1;
+      break;
+    default:
+      /* Advance in block */
       if(newpos>=tracker->curpattern->length) {
 	newpos=0;
 	if(editor.mode==MODE_PLAY_SONG)
 	  player_next_playseq();
-	player_refresh_playseq_and_block();
-	gdk_threads_enter();
-	tracker_set_pattern(tracker, editor.song->blocks[editor.block]);
-	gui_info_refresh();
-	gui_blocklist_refresh();
-	gui_playseq_refresh();
-	gdk_threads_leave();
+	changeblock=1;
       }
-      gdk_threads_enter();
-      tracker_set_patpos(tracker, newpos);
-      gettimeofday(&tod, NULL);
-      gui_timer_refresh((int)(editor.playedsofar.tv_sec+tod.tv_sec-editor.playingstarted.tv_sec));
-      gdk_threads_leave();
+      break;
     }
-    break;
+
+    gdk_threads_enter();
+    if(changeblock==1) {
+      player_refresh_playseq_and_block();
+      tracker_set_pattern(tracker, editor.song->blocks[editor.block]);
+      gui_info_refresh();
+      gui_blocklist_refresh();
+      gui_playseq_refresh();
+    }
+    tracker_set_patpos(tracker, newpos);
+    gettimeofday(&tod, NULL);
+    gui_timer_refresh((int)(editor.playedsofar.tv_sec+tod.tv_sec-editor.playingstarted.tv_sec));
+    gdk_threads_leave();
   }
 }
 
@@ -249,8 +347,9 @@ void player_next_section() {
 void player_play_note(unsigned short instrument, unsigned char note,
 		      unsigned char volume, unsigned char track) {
   /* Stop currently playing note */
-  if(editor.trackstatus[track*4+1]!=-1)
-    midi_note_off(editor.trackstatus[track*4], editor.trackstatus[track*4+1],
+  if(editor.trackstatus[track*4+TRACK_STATUS_NOTE]!=-1)
+    midi_note_off(editor.trackstatus[track*4+TRACK_STATUS_MIDI_CHANNEL],
+		  editor.trackstatus[track*4+TRACK_STATUS_NOTE],
 		  127);
 
   /* Don't play a note if the instrument does not exist */
@@ -260,21 +359,22 @@ void player_play_note(unsigned short instrument, unsigned char note,
     note+=editor.song->instruments[instrument]->transpose;
 
     /* Set new track status (channel, note, volume, time) */
-    editor.trackstatus[track*4]=editor.song->instruments[instrument]->midichannel;
-    editor.trackstatus[track*4+1]=note;
-    editor.trackstatus[track*4+2]=editor.song->instruments[instrument]->defaultvelocity*volume/127*(editor.song->trackvolumes[track]&127)/127*((255-editor.song->trackvolumes[track])&128)/128*editor.song->mastervolume/127;
+    editor.trackstatus[track*4+TRACK_STATUS_MIDI_CHANNEL]=editor.song->instruments[instrument]->midichannel;
+    editor.trackstatus[track*4+TRACK_STATUS_NOTE]=note;
+    editor.trackstatus[track*4+TRACK_STATUS_VOLUME]=editor.song->instruments[instrument]->defaultvelocity*volume/127*(editor.song->trackvolumes[track]&127)/127*((255-editor.song->trackvolumes[track])&128)/128*editor.song->mastervolume/127;
     if(editor.song->instruments[instrument]->hold>0)
-      editor.trackstatus[track*4+3]=editor.song->instruments[instrument]->hold;
+      editor.trackstatus[track*4+TRACK_STATUS_HOLD]=editor.song->instruments[instrument]->hold;
     else
-      editor.trackstatus[track*4+3]=-1;
+      editor.trackstatus[track*4+TRACK_STATUS_HOLD]=-1;
 
     /* Make sure the volume isn't too large */
-    if(editor.trackstatus[track*4+2]<0)
-      editor.trackstatus[track*4+2]=127;
+    if(editor.trackstatus[track*4+TRACK_STATUS_VOLUME]<0)
+      editor.trackstatus[track*4+TRACK_STATUS_VOLUME]=127;
     
     /* Play note */
-    midi_note_on(editor.trackstatus[track*4], editor.trackstatus[track*4+1],
-		 editor.trackstatus[track*4+2]);
+    midi_note_on(editor.trackstatus[track*4+TRACK_STATUS_MIDI_CHANNEL],
+		 editor.trackstatus[track*4+TRACK_STATUS_NOTE],
+		 editor.trackstatus[track*4+TRACK_STATUS_VOLUME]);
   }
 }
 
@@ -290,6 +390,10 @@ void player_stop_notes() {
       editor.trackstatus[i*4+3]=-1;
     }
   }
+
+  /* Reset MIDI controller values to 0x40 (can't be horribly wrong...) */
+  for(i=0; i<16*VALUES; i++)
+    editor.midicontrollervalues[i]=0x40;
 }
 
 /* Stops all notes playing at the moment */
