@@ -25,6 +25,8 @@
 #include "editor.h"
 #include "player.h"
 
+char *xmlencode(unsigned char *, int);
+
 extern struct editor editor;
 
 /* Allocates a new song structure and initializes it to default values */
@@ -267,6 +269,57 @@ void song_delete_section(struct song *song, int pos) {
   }
 }
 
+/* Inserts a new sysex in the sysex array in the given position */
+void song_insert_sysex(struct song *song, int pos) {
+  int i;
+
+  /* Check sysex existence */
+  if(pos<0)
+    pos=0;
+  else if(pos>song->numsysexes)
+    pos=song->numsysexes;
+
+  song->numsysexes++;
+  /* Reallocate the sysex array */
+  song->sysexes=(struct sysex **)realloc(song->sysexes, song->numsysexes*sizeof(struct sysex *));
+
+  /* Move rest of the sysexes onwards */
+  for(i=song->numsysexes-1; i>pos; i--)
+    song->sysexes[i]=song->sysexes[i-1];
+
+  /* Insert a new sysex similar to the current sysex */
+  song->sysexes[pos]=(struct sysex *)calloc(1, sizeof(struct sysex));
+}
+
+/* Deletes a sysex from the given position of the sysex array */
+void song_delete_sysex(struct song *song, int pos) {
+  int i;
+
+  /* Check sysex existence */
+  if(pos<0)
+    pos=0;
+  else if(pos>=song->numsysexes)
+    pos=song->numsysexes-1;
+
+  /* Don't delete inexisting sysexes */
+  if(song->numsysexes>0) {
+    /* Free the sysex in question */
+    if(song->sysexes[pos]->data!=NULL)
+      free(song->sysexes[pos]->data);
+    if(song->sysexes[pos]->name!=NULL)
+      free(song->sysexes[pos]->name);
+    free(song->sysexes[pos]);
+    song->numsysexes--;
+    
+    /* Move rest of the sysexes backwards */
+    for(i=pos; i<song->numsysexes; i++)
+      song->sysexes[i]=song->sysexes[i+1];
+    
+    /* Reallocate the sysex array */
+    song->sysexes=(struct sysex **)realloc(song->sysexes, song->numsysexes*sizeof(struct sysex *));
+  }
+}
+
 /* Sets a section in the given position to point somewhere */
 void song_set_section(struct song *song, int pos, int playseq) {
   if(playseq>=0 && playseq<song->numplayseqs)
@@ -342,7 +395,7 @@ struct song *song_convert_MMD2(struct MMD2 *mmd) {
   effectconversion[2]=EFFECT_PITCH_WHEEL;
   effectconversion[3]=EFFECT_PITCH_WHEEL;
   effectconversion[4]=EFFECT_NOT_DEFINED;
-  effectconversion[5]=EFFECT_MIDI_COMMAND;
+  effectconversion[5]=EFFECT_NOT_DEFINED;
   effectconversion[6]=EFFECT_NOT_DEFINED;
   effectconversion[7]=EFFECT_NOT_DEFINED;
   effectconversion[8]=EFFECT_NOT_DEFINED;
@@ -353,7 +406,7 @@ struct song *song_convert_MMD2(struct MMD2 *mmd) {
   effectconversion[13]=EFFECT_CHANNEL_PRESSURE;
   effectconversion[14]=EFFECT_NOT_DEFINED;
   effectconversion[15]=EFFECT_TEMPO;
-  effectconversion[16]=EFFECT_NOT_DEFINED;
+  effectconversion[16]=EFFECT_SEND_SYSEX;
   effectconversion[17]=EFFECT_NOT_DEFINED;
   effectconversion[18]=EFFECT_NOT_DEFINED;
   effectconversion[19]=EFFECT_NOT_DEFINED;
@@ -368,7 +421,7 @@ struct song *song_convert_MMD2(struct MMD2 *mmd) {
   effectconversion[28]=EFFECT_NOT_DEFINED;
   effectconversion[29]=EFFECT_NOT_DEFINED;
   effectconversion[30]=EFFECT_NOT_DEFINED;
-  effectconversion[31]=EFFECT_NOT_DEFINED;
+  effectconversion[31]=EFFECT_NOTE_DELAY;
 
   song=(struct song *)calloc(1, sizeof(struct song));
   if(mmd->expdata && mmd->expdata->songname)
@@ -425,14 +478,19 @@ struct song *song_convert_MMD2(struct MMD2 *mmd) {
 	  block->effects[(j*block->tracks+k)*2]=effectconversion[notes[(j*block->tracks+k)*4+2]];
 	  block->effects[(j*block->tracks+k)*2+1]=notes[(j*block->tracks+k)*4+3];
 	  /* Effect values may need special handling */
-	  /* Fix volume range to 0..127 */
 	  switch(block->effects[(j*block->tracks+k)*2]) {
 	  case EFFECT_VELOCITY:
+	    /* Fix volume range to 0..127 */
 	    if(block->effects[(j*block->tracks+k)*2+1]==64)
 	      block->effects[(j*block->tracks+k)*2+1]=127;
 	    else
 	      block->effects[(j*block->tracks+k)*2+1]*=2;
 	    break;
+	  case EFFECT_NOTE_DELAY:
+	    /* Command 1F in MMD is both note delay and repeat */
+	    /* FIX: now only note delay (upper nibble) is supported */
+	    block->effects[(j*block->tracks+k)*2+1]=block->effects[(j*block->tracks+k)*2+1]>>4;
+	    break;
 	  }
 	}
 
@@ -463,8 +521,10 @@ struct song *song_convert_MMD2(struct MMD2 *mmd) {
 						   sizeof(struct instrument *));
     for(i=0; i<song->numinstruments; i++) {
       song->instruments[i]=instrument_alloc();
-      song->instruments[i]->name=(char *)strdup(mmd->expdata->iinfo[i].name);
-      song->instruments[i]->hold=mmd->expdata->exp_smp[i].hold;
+      if(mmd->expdata) {
+	song->instruments[i]->name=(char *)strdup(mmd->expdata->iinfo[i].name);
+	song->instruments[i]->hold=mmd->expdata->exp_smp[i].hold;
+      }
       song->instruments[i]->midichannel=mmd->song->sample[i].midich-1;
       song->instruments[i]->midipreset=mmd->song->sample[i].midipreset;
       song->instruments[i]->transpose=mmd->song->sample[i].strans;
@@ -473,6 +533,8 @@ struct song *song_convert_MMD2(struct MMD2 *mmd) {
       else
 	song->instruments[i]->defaultvelocity=mmd->song->sample[i].svol*2;
     }
+
+    /* Track volumes */
     song->maxtracks=mmd->song->numtracks;
     song->trackvolumes=(char *)calloc(song->maxtracks, sizeof(char));
     for(i=0; i<song->maxtracks; i++) {
@@ -485,6 +547,28 @@ struct song *song_convert_MMD2(struct MMD2 *mmd) {
       song->mastervolume=mmd->song->mastervol*2;
     else
       song->mastervolume=127;
+
+    /* SysEX dumps */
+    if(mmd->expdata && mmd->expdata->dumps) {
+      /* Is this really the only way to do this? I know, this is horrible but
+       * this is what the lame MMD spec says */
+      struct MMDDump **dumps=(struct MMDDump **)((char *)mmd->expdata->dumps+
+						 sizeof(struct MMDDumpData));
+
+      song->numsysexes=mmd->expdata->dumps->numdumps;
+      song->sysexes=(struct sysex **)calloc(song->numsysexes,
+						   sizeof(struct sysex *));
+      for(i=0; i<song->numsysexes; i++) {
+	song->sysexes[i]=(struct sysex *)calloc(1, sizeof(struct sysex));
+	song->sysexes[i]->length=dumps[i]->length;
+	song->sysexes[i]->data=(unsigned char *)calloc(song->sysexes[i]->length, sizeof(unsigned char));
+	memcpy(song->sysexes[i]->data, dumps[i]->data,
+	       song->sysexes[i]->length);
+
+	if(dumps[i]->ext_len>=20)
+	  song->sysexes[i]->name=strdup(dumps[i]->name);
+      }
+    }
   } else {
     /* Nothing can be done in this case... should never happen though */
     free(song);
@@ -626,6 +710,23 @@ struct song *song_parse(xmlDocPtr doc, xmlNsPtr ns, xmlNodePtr cur) {
 	    }
 	  }
 	  
+	  temp=temp->next;
+	}
+      } else if((!xmlStrcmp(cur->name, "sysexes")) && (cur->ns==ns)) {
+	xmlNodePtr temp;
+
+	/* Parse and add all SysEx elements */
+	temp=cur->xmlChildrenNode;
+	while(temp!=NULL) {
+	  if(!(xmlIsBlankNode(temp))) {
+	    struct sysex *sysex=sysex_parse(doc, ns, temp);
+	    if(sysex!=NULL) {
+	      song->numsysexes++;
+	      song->sysexes=realloc(song->sysexes, song->numsysexes*sizeof(struct sysex *));
+	      song->sysexes[song->numsysexes-1]=sysex;
+	    }
+	  }
+	  
 	  temp=temp->next;
 	}
       }
@@ -711,6 +812,15 @@ void song_save(struct song *song, char *filename) {
   lb=xmlNewText("\n\n");
   xmlAddChild(doc->xmlRootNode, lb);
 
+  node=xmlNewChild(doc->xmlRootNode, NULL, "sysexes", NULL);
+  lb=xmlNewText("\n");
+  xmlAddChild(node, lb);
+  /* Add all SysEx messages */
+  for(i=0; i<song->numsysexes; i++)
+    sysex_save(song->sysexes[i], node);
+  lb=xmlNewText("\n\n");
+  xmlAddChild(doc->xmlRootNode, lb);
+
   /* Use stdout if no filename is specified */
   if(filename==NULL)
     filename="-";
@@ -718,4 +828,3 @@ void song_save(struct song *song, char *filename) {
   xmlSaveFile(filename, doc);
   xmlFreeDoc(doc);
 }
-
